@@ -15,16 +15,32 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-import sys, gc
+import sys, os
+
+# Find common code relative in file system. Not using packages for test stuff.
+sys.path.insert(
+    0,
+    os.path.normpath(
+        os.path.join(
+            os.path.dirname(os.path.abspath(__file__)),
+            ".."
+        )
+    )
+)
+from test_common import (
+    executeReferenceChecked,
+    checkReferenceCount,
+    my_print,
+)
 
 if not hasattr(sys, "gettotalrefcount"):
-    print("Warning, using non-debug Python makes this test ineffective.")
+    my_print("Warning, using non-debug Python makes this test ineffective.")
     sys.gettotalrefcount = lambda : 0
 
-gc.disable()
+x = 17
 
 def simpleFunction1():
-   return 1
+    return 1
 
 def simpleFunction2():
     y = 3 * x
@@ -96,7 +112,7 @@ def simpleFunction13(a = 1*2):
 def simpleFunction14p(x):
     try:
         simpleFunction14p(1,1)
-    except TypeError, e:
+    except TypeError as e:
         pass
 
     try:
@@ -149,14 +165,14 @@ def simpleFunction18():
 def simpleFunction19():
     lam = lambda l : l+1
 
-    return lam( 9 ), lam
+    return lam(9), lam
 
 
 def simpleFunction20():
     try:
         a = []
         a[1]
-    except IndexError, e:
+    except IndexError as e:
         pass
 
 
@@ -233,73 +249,73 @@ def simpleFunction30():
         yield 3
 
 def simpleFunction31():
-   def generatorFunction():
-      yield 1
-      yield 2
-      yield 3
+    def generatorFunction():
+        yield 1
+        yield 2
+        yield 3
 
-   a = []
+    a = []
 
-   for y in generatorFunction():
-      a.append( y )
+    for y in generatorFunction():
+        a.append( y )
 
-   for z in generatorFunction():
-      a.append( z )
+    for z in generatorFunction():
+        a.append( z )
 
 
 def simpleFunction32():
-   def generatorFunction():
-      yield 1
+    def generatorFunction():
+        yield 1
 
-   gen = generatorFunction()
-   gen.next()
+    gen = generatorFunction()
+    next(gen)
 
 def simpleFunction33():
-   def generatorFunction():
-      a = 1
+    def generatorFunction():
+        a = 1
 
-      yield a
+        yield a
 
-   a = []
+    a = []
 
-   for y in generatorFunction():
-      a.append( y )
+    for y in generatorFunction():
+        a.append( y )
 
 
 def simpleFunction34():
-   try:
-      raise ValueError
-   except:
-      pass
+    try:
+        raise ValueError
+    except:
+        pass
 
 def simpleFunction35():
-   try:
-      raise ValueError(1,2,3)
-   except:
-      pass
+    try:
+        raise ValueError(1,2,3)
+    except:
+        pass
 
 
 def simpleFunction36():
-   try:
-      raise TypeError, (3,x,x,x)
-   except TypeError:
-      pass
+    try:
+        raise (TypeError, (3,x,x,x))
+    except TypeError:
+        pass
 
 def simpleFunction37():
-   l = [ 1, 2, 3 ]
+    l = [ 1, 2, 3 ]
 
-   try:
-      a, b = l
-   except ValueError:
-      pass
+    try:
+        a, b = l
+    except ValueError:
+        pass
 
 
 def simpleFunction38():
-   class Base:
-      pass
+    class Base:
+        pass
 
-   class Parent(Base):
-      pass
+    class Parent(Base):
+        pass
 
 def simpleFunction39():
     class Parent(object):
@@ -328,122 +344,98 @@ def simpleFunction43():
 
     a.b = 1
 
-def simpleFunction44():
-    def nested_args_function((a,b), c):
-        return a, b, c
-
-    nested_args_function( ( 1, 2 ), 3 )
-
-def simpleFunction45():
-    def nested_args_function((a,b), c):
-        return a, b, c
-
-    try:
-        nested_args_function( ( 1, ), 3 )
-    except ValueError:
-        pass
-
-def simpleFunction46():
-    def nested_args_function((a,b), c):
-        return a, b, c
-
-    try:
-        nested_args_function(( 1, 2, 3 ), 3)
-    except ValueError:
-        pass
-
 def simpleFunction47():
-   def reraisy():
-      def raisingFunction():
-         raise ValueError(3)
+    def reraisy():
+        def raisingFunction():
+            raise ValueError(3)
 
-      def reraiser():
-         raise
+        def reraiser():
+            raise
 
-      try:
-         raisingFunction()
-      except:
-         reraiser()
+        try:
+            raisingFunction()
+        except:
+            reraiser()
 
-   try:
-      reraisy()
-   except:
-      pass
+    try:
+        reraisy()
+    except:
+        pass
 
 def simpleFunction48():
-   class BlockExceptions:
-      def __enter__(self):
-         pass
-      def __exit__( self, exc, val, tb):
-         return True
+    class BlockExceptions:
+        def __enter__(self):
+            pass
+        def __exit__( self, exc, val, tb):
+            return True
 
-   with BlockExceptions():
-      raise ValueError()
+    with BlockExceptions():
+        raise ValueError()
 
 template = "lala %s lala"
 
 def simpleFunction49():
-   c = 3
-   d = 4
+    c = 3
+    d = 4
 
-   a = x, y = b,e = (c,d)
+    a = x, y = b,e = (c,d)
 
 b = range(10)
 
 def simpleFunction50():
-   def getF():
-      def f():
-         for i in b:
-            yield i
+    def getF():
+        def f():
+            for i in b:
+                yield i
 
-      return f
+        return f
 
-   f = getF()
+    f = getF()
 
-   for x in range( 2 ):
-      r = list( f() )
+    for x in range( 2 ):
+        r = list( f() )
 
 def simpleFunction51():
-   g = ( x for x in range(9) )
+    g = ( x for x in range(9) )
 
-   try:
-      g.throw( ValueError, 9 )
-   except ValueError, e:
-      pass
+    try:
+        g.throw( ValueError, 9 )
+    except ValueError as e:
+        pass
 
 def simpleFunction52():
-   g = ( x for x in range(9) )
+    g = ( x for x in range(9) )
 
-   try:
-      g.throw( ValueError( 9 ) )
-   except ValueError, e:
-      pass
+    try:
+        g.throw( ValueError(9) )
+    except ValueError as e:
+        pass
 
 def simpleFunction53():
     g = ( x for x in range(9) )
 
     try:
-        g.send( 9 )
-    except TypeError, e:
+        g.send(9)
+    except TypeError as e:
         pass
 
 def simpleFunction54():
     g = ( x for x in range(9) )
-    g.next()
+    next(g)
 
     try:
-       g.send( 9 )
-    except TypeError, e:
+        g.send(9)
+    except TypeError as e:
         pass
 
 
 def simpleFunction55():
-   g = ( x for x in range(9) )
+    g = ( x for x in range(9) )
 
-   try:
-      g.close()
-   except ValueError, e:
-      pass
+    try:
+        g.close()
+    except ValueError as e:
+        pass
 
 def simpleFunction56():
     def f():
@@ -489,30 +481,12 @@ def simpleFunction59():
         return a / b
 
 
-def simpleFunction60():
-    try:
-        raise ValueError(1,2,3), ValueError(1,2,3)
-    except Exception:
-        pass
-
-def simpleFunction61():
-    try:
-        raise ValueError, 2, None
-    except Exception:
-        pass
-
-def simpleFunction62():
-    try:
-        raise ValueError, 2, 3
-    except Exception:
-        pass
-
 class X:
     def __del__(self):
         # Super used to reference leak.
         x = super()
 
-        raise ValueError, ValueError(1)
+        raise ValueError(1)
 
 def simpleFunction63():
     def superUser():
@@ -559,7 +533,7 @@ def simpleFunction68():
 def simpleFunction69():
     pools = [ tuple() ]
     g = ((len(pool) == 0,) for pool in pools)
-    g.next()
+    next(g)
 
 def simpleFunction70():
     def gen():
@@ -615,7 +589,7 @@ def simpleFunction74():
             yield
 
     g = raising_gen()
-    g.next()
+    next(g)
 
     try:
         g.throw(RuntimeError())
@@ -657,179 +631,20 @@ def simpleFunction77():
     exec(a)
 
 
-x = 17
+# These need stderr to be wrapped.
+tests_stderr = (63,)
 
-m1 = {}
-m2 = {}
+# Disabled tests
+tests_skipped = {
+   56 : "Functions with closure to themselves are not yet released.",
+   76 : "Functions with closure to themselves are not yet released.",
+}
 
-def snapObjRefCntMap(before):
-   if before:
-      global m1
-      m = m1
-   else:
-      global m2
-      m = m2
+result = executeReferenceChecked(
+    prefix        = "simpleFunction",
+    names         = globals(),
+    tests_skipped = tests_skipped,
+    tests_stderr  = tests_stderr
+)
 
-   for x in gc.get_objects():
-      if x is m1:
-         continue
-
-      if x is m2:
-         continue
-
-      m[ str( x ) ] = sys.getrefcount( x )
-
-
-def checkReferenceCount(checked_function, max_rounds = 10):
-   assert sys.exc_info() == ( None, None, None ), sys.exc_info()
-
-   print checked_function.func_name + ":",
-
-   ref_count1 = 17
-   ref_count2 = 17
-
-   explain = False
-
-   for count in range( max_rounds ):
-      x1 = 0
-      x2 = 0
-
-      gc.collect()
-      ref_count1 = sys.gettotalrefcount()
-
-      if explain and count == max_rounds - 1:
-         snapObjRefCntMap( True )
-
-      checked_function()
-
-      assert sys.exc_info() == ( None, None, None ), sys.exc_info()
-
-      gc.collect()
-
-      if explain and count == max_rounds - 1:
-         snapObjRefCntMap( False )
-
-      ref_count2 = sys.gettotalrefcount()
-
-      if ref_count1 == ref_count2:
-         print "PASSED"
-         break
-
-      # print count, ref_count1, ref_count2
-   else:
-      print "FAILED", ref_count1, ref_count2, "leaked", ref_count2 - ref_count1
-
-      if explain:
-         assert m1
-         assert m2
-
-         for key in m1.keys():
-            if key not in m2:
-               print "*" * 80
-               print key
-            elif m1[key] != m2[key]:
-               print "*" * 80
-               print key
-            else:
-               pass
-               # print m1[key]
-
-   assert sys.exc_info() == ( None, None, None ), sys.exc_info()
-
-   gc.collect()
-
-
-checkReferenceCount( simpleFunction1 )
-checkReferenceCount( simpleFunction2 )
-checkReferenceCount( simpleFunction3 )
-checkReferenceCount( simpleFunction4 )
-checkReferenceCount( simpleFunction5 )
-checkReferenceCount( simpleFunction6 )
-checkReferenceCount( simpleFunction7 )
-checkReferenceCount( simpleFunction8 )
-checkReferenceCount( simpleFunction9 )
-checkReferenceCount( simpleFunction10 )
-checkReferenceCount( simpleFunction11 )
-checkReferenceCount( simpleFunction12 )
-checkReferenceCount( simpleFunction13 )
-checkReferenceCount( simpleFunction14 )
-checkReferenceCount( simpleFunction15 )
-checkReferenceCount( simpleFunction16 )
-checkReferenceCount( simpleFunction17 )
-checkReferenceCount( simpleFunction18 )
-checkReferenceCount( simpleFunction19 )
-checkReferenceCount( simpleFunction20 )
-checkReferenceCount( simpleFunction21 )
-checkReferenceCount( simpleFunction22 )
-checkReferenceCount( simpleFunction23 )
-checkReferenceCount( simpleFunction24 )
-checkReferenceCount( simpleFunction25 )
-checkReferenceCount( simpleFunction26 )
-checkReferenceCount( simpleFunction27 )
-checkReferenceCount( simpleFunction28 )
-checkReferenceCount( simpleFunction29 )
-checkReferenceCount( simpleFunction30 )
-checkReferenceCount( simpleFunction31 )
-checkReferenceCount( simpleFunction32 )
-checkReferenceCount( simpleFunction33 )
-checkReferenceCount( simpleFunction34 )
-checkReferenceCount( simpleFunction35 )
-checkReferenceCount( simpleFunction36 )
-checkReferenceCount( simpleFunction37 )
-checkReferenceCount( simpleFunction38 )
-checkReferenceCount( simpleFunction39 )
-checkReferenceCount( simpleFunction40 )
-checkReferenceCount( simpleFunction41 )
-checkReferenceCount( simpleFunction42 )
-checkReferenceCount( simpleFunction43 )
-checkReferenceCount( simpleFunction44 )
-checkReferenceCount( simpleFunction45 )
-checkReferenceCount( simpleFunction46 )
-checkReferenceCount( simpleFunction47 )
-checkReferenceCount( simpleFunction48 )
-checkReferenceCount( simpleFunction49 )
-checkReferenceCount( simpleFunction50 )
-checkReferenceCount( simpleFunction51 )
-checkReferenceCount( simpleFunction52 )
-checkReferenceCount( simpleFunction53 )
-checkReferenceCount( simpleFunction54 )
-checkReferenceCount( simpleFunction55 )
-# TODO: The function taking a closure of itself, causes a reference leak, that
-# we accept for now.
-# checkReferenceCount( simpleFunction56 )
-checkReferenceCount( simpleFunction57 )
-checkReferenceCount( simpleFunction58 )
-checkReferenceCount( simpleFunction59 )
-checkReferenceCount( simpleFunction60 )
-checkReferenceCount( simpleFunction61 )
-checkReferenceCount( simpleFunction62 )
-
-# Avoid unraisable output.
-old_stderr = sys.stderr
-try:
-   sys.stderr = open( "/dev/null", "wb" )
-except Exception: # Windows
-    checkReferenceCount(simpleFunction63)
-else:
-    checkReferenceCount(simpleFunction63)
-
-    new_stderr = sys.stderr
-    sys.stderr = old_stderr
-    new_stderr.close()
-
-checkReferenceCount(simpleFunction64)
-checkReferenceCount(simpleFunction65)
-checkReferenceCount(simpleFunction66)
-checkReferenceCount(simpleFunction67)
-checkReferenceCount(simpleFunction68)
-checkReferenceCount(simpleFunction69)
-checkReferenceCount(simpleFunction70)
-checkReferenceCount(simpleFunction71)
-checkReferenceCount(simpleFunction72)
-checkReferenceCount(simpleFunction73)
-checkReferenceCount(simpleFunction74)
-checkReferenceCount(simpleFunction75)
-# TODO: The class taking a closure of itself, causes a reference leak, that
-# we accept for now.
-# checkReferenceCount( simpleFunction76 )
-checkReferenceCount(simpleFunction77)
+sys.exit(0 if result else 1)
\ No newline at end of file
